#!/usr/bin/python2

try:
    import utilities
    import os
    import re
    import sys 
    import shlex
    import select
    import subprocess
    import fcntl
    import socket
    import time
    import shutil
    from pprint import pprint

except ImportError as e:
    utilities.printError(e, raiseError=True)


class HWCert:
    def __init__(self):
        self.rhcertCmd = "/usr/bin/rhcert-cli "
        self.rhcertdCmd = "/usr/bin/rhcertd "
        self.rhcertuCmd = "/usr/bin/rhcert-util "
        self.hwTests = [ 'self_check', 'audio', 'cdrom', 'dvd',  'usb2', 'usb3',
                        'kdump', 'memory', 'core', 'cpuscaling', 'fv_core', 'fv_memory',
                        'hw_profiler', 'SATA', 'video', 'info', 'sosreport', 'sw_profiler',
                        'STORAGE', 'SAS', 'SATA_SSD', '100MegEthernet', '1GigEthernet',
                        '2.5GigEthernet', '5GigEthernet', '10GigEthernet', '20GigEthernet',
                        '25GigEthernet', '40GigEthernet', '50GigEthernet', '100GigEthernet',
			'realtime', 'fv_realtime', 'fv_cpu_pinning' ]

        self.printStats = {}
        self.planDetails = {}
        self.planArgs = {}
        self.checkPlanned = []
        self.checkPlannedRaw = ""
        self.runDetails = {}
        self.couldNotPlan = []
        self.alwaysPlanIfUnplanned = ['core', 'memory', 'fv_core', 'fv_memory', 'fv_cpu_pinning']

    def setup_info(self, verify=True):
        retStatus = True
        err = ""
        setupInfo = {}
        hostName = utilities.runShellCommand("hostname -f", rhcertCommand=False)[1]
        osVer = utilities.runShellCommand("cat /etc/redhat-release", rhcertCommand=False)[1]
        archType = utilities.runShellCommand("uname -m", rhcertCommand=False)[1]
        kernVer = utilities.runShellCommand("uname -r", rhcertCommand=False)[1]
        rhcertPackages = utilities.runShellCommand("rpm -qa 'redhat-certification*'")[1]
        if rhcertPackages:
            if isinstance(rhcertPackages, bytes):
                pkgList = filter(None, rhcertPackages.decode().split('\n'))
            else:
                pkgList = filter(None, rhcertPackages.split('\n'))
            # test
            hwPkgs = [ i for i in pkgList if 'hardware' in i or 'backend' in i ]
        else:
            retStatus = False
            err = "Required Certification RPM's are not installed"
            return retStatus, setupInfo, err

        # version, build via RPM
        if len(hwPkgs) < 2:
            rhcertVer = hwPkgs[0].split('-')[3].strip()
        else:
            rhcertVer = hwPkgs[1].split('-')[3].strip()

        rhcertBuild = hwPkgs[0].split('-')[4].split('el')[0].rstrip('.')

        setupInfo['hostname'] = hostName.decode().strip() if isinstance(hostName, bytes) else hostName.strip()
        setupInfo['os_version'] = osVer.decode().strip() if isinstance(osVer, bytes) else osVer.strip()
        setupInfo['arch'] = archType.decode().strip() if isinstance(archType, bytes) else archType.strip()
        setupInfo['kernel'] = kernVer.decode().strip() if isinstance(kernVer, bytes) else kernVer.strip()
        setupInfo['rhcert_version'] = rhcertVer
        setupInfo['rhcert_build'] = rhcertBuild
        print('setupInfo : {}'.format(setupInfo))
        return retStatus, setupInfo, err

    def daemon_status(self, verify=True):
        psOutput = utilities.runShellCommand("pgrep rhcertd", rhcertCommand=False)[1]
        # default running
        retStatus = False
        if verify:
            if psOutput:
                # if still running
                retStatus = True

        return retStatus

    def daemon_stop(self, verify=True):
        stopCmd = "stop"
        Cmd = self.rhcertdCmd + stopCmd
        (exitStatus, out, err) = utilities.runShellCommand(
                                    Cmd, rhcertCommand=False
                                )

        retStatus = True
        if verify:
            if self.daemon_status():
                print("Daemon not stopped, please try again")
                sys.exit(1)

        return retStatus


    def daemon_start(self, verify=True):
        startCmd = "start"
        Cmd = self.rhcertdCmd + startCmd
        (exitStatus, out, err) = utilities.runShellCommand(
                                    Cmd, rhcertCommand=False
                                )

        retStatus = True
        if verify:
            if not self.daemon_status():
                retStatus = False

        return retStatus

    def cleanup(self, verify=True):
        rhcertLockfile = "/var/lock/subsys/rhcert"
        psOutput = utilities.runShellCommand("pgrep rhcert-cli", rhcertCommand=False)[1]
        retStatus = True
        if not psOutput and not os.path.isfile(rhcertLockfile):
            return retStatus

        psOutput = psOutput.split()
        if verify:
            if os.path.isfile(rhcertLockfile):
                try:
                    t = os.remove(rhcertLockfile)
                except:
                    print('Unable to remove file : {}'.format(rhcertLockfile))
                    pass
            try:
                for p in psOutput:
                    utilities.runShellCommand("kill -9 {}".format(p))
            except:
                print('Unable to kill rhcert-cli processes')
                retStatus = False

        return retStatus

    def clean_local_storage(self, verify=True):
        oldiNode = utilities.runShellCommand("ls -ldi /var/www/rhcert/store/", rhcertCommand=False)[1].split()[0].strip()
        oldPids = utilities.runShellCommand("pgrep httpd", rhcertCommand=False)[1].split()
        oldPids.sort()
        cleanCmd = "clean local storage"
        Cmd = self.rhcertuCmd + cleanCmd
        (exitStatus, out, err) = utilities.runShellCommand(
                                    Cmd, rhcertCommand=True
                                )

        retStatus = True
        newiNode = utilities.runShellCommand("ls -ldi /var/www/rhcert/store/", rhcertCommand=False)[1].split()[0].strip()
        newPids = utilities.runShellCommand("pgrep httpd", rhcertCommand=False)[1].split()
        newPids.sort()
        # verify if directory '/var/www/rhcert/store' was removed and recreated in the backend
        # verify pid of httpd was changed since process was restarted
        if verify:
            if oldiNode == newiNode:
                retStatus = False
            if oldPids == newPids:
                retStatus = False

        return retStatus

    def plan(self, verify=False):
        ret = self.clean()
        planCmd = "plan"
        Cmd = self.rhcertCmd +  planCmd
        (exitStatus, out, err) = utilities.runShellCommand(
                                    Cmd, rhcertCommand=True
                                )

        retStatus = True
        if exitStatus != 0:
            print("{} did not complete successfully.".format(planCmd))
            #assert False, "Unable to parse test plan information from rhcert-cli."
            retStatus = False
    
        # check for tests that could not be planned due to setup issue and manually plan them
        # check CERTPX-6180 for more details, introduce self.couldNotPlan list and populate it
        for line in out.split('\n'):
            if 'Could not plan test' in line:
                self.couldNotPlan.append(line.split()[-1].split('/')[-1])
        print('self.couldNotPlan : {}'.format(self.couldNotPlan))
        self.doprint()
        self.checkPlanned.extend(self.couldNotPlan)
        return retStatus, self.checkPlanned, self.planArgs

    def run(self, testname, device=False, devicename='dummy', debug=False, verify=False):
        ret = self.clean()

        if testname in self.couldNotPlan and testname in self.alwaysPlanIfUnplanned:
            manualplanCmd = "plan --add --test " + testname
            Cmd = self.rhcertCmd + manualplanCmd
            (exitStatus, out, err) = utilities.runShellCommand(
                                        Cmd, rhcertCommand=True
                                    )

        if devicename != 'dummy':
            devicename = devicename.decode().strip() if isinstance(devicename, bytes) else devicename.strip()
        runCmd = "run"
        Cmd = self.rhcertCmd + runCmd + ' --test ' + testname
        if device and devicename != 'dummy':
            Cmd = self.rhcertCmd + runCmd + ' --test ' + testname + ' --device ' + str(devicename)
        if debug:
            Cmd = Cmd + ' --debug high'
        (exitStatus, out, err) = utilities.runShellCommand(
                                    Cmd, rhcertCommand=True
                                )
        retStatus = True
        self.doprint()
        ret = self.report(testname)
        if not ret:
            retStatus = False
        # save results
        self.save(testname)
        return (retStatus, out.strip(), err)

    def doprint(self, verify=False):
        printCmd = "print"
        Cmd = self.rhcertCmd + printCmd
        (exitStatus, out, err) = utilities.runShellCommand(
                                    Cmd, rhcertCommand=False
                                )
        retStatus = True
        # verify if print results are empty and return
        if out.strip() == 'No test results or plan to print':
            retStatus = False
            return retStatus

        trunFlag = False
        tplanFlag = False
        tCount = 1
        for i in out.decode().split('\n'):
            if not i:
                continue
            # need to handle only planned TC's where Test run is not available, only plan is available
            if 'Test Plan' in i:
                tplanFlag = True
            if 'Tests:' in i:
                tmp = i
                self.printStats['planned'] = tmp.split(',')[0].strip().split(':')[1].split()[0]
                self.printStats['run']     = tmp.split(',')[1].strip().strip().split()[0]
                self.printStats['passed']  = tmp.split(',')[2].strip().strip().split()[0]
                self.printStats['failed']  = tmp.split(',')[3].strip().strip().split()[0]
            if 'Test Run' in i:
                trunFlag = True
                continue
            if trunFlag:
                if '----' in i:
                    continue
                if i.split()[0] in self.hwTests:
                    tmp = [ j.strip() for j in i.split() ]
                    self.runDetails['testcase' + str(tCount)] = tmp
                    tCount = tCount + 1
                    if tmp[0] == 'sosreport':
                        break
            if tplanFlag:
                self.checkPlannedRaw = self.checkPlannedRaw + i + "\n"
                if '----' in i:
                    continue
                if i.split()[0].split('/')[-1] in self.hwTests:
                    tmp = [ j.strip() for j in i.split() ]
                    self.planDetails['testcase' + str(tCount)] = tmp
                    tCount = tCount + 1

        for k, v in self.planDetails.items():
            tname = v[0].split('/')[-1]
            if len(v) == 1:
                self.planArgs[tname] = ''
            else:
                if not tname in self.planArgs.keys():
                    self.planArgs[tname] = []
                self.planArgs[tname].append(v[1:])
        ptmp = []
        for k, v in self.planDetails.items():
            ptmp.append(v[0].split('/')[-1])
        self.checkPlanned = list(set(ptmp))
        return (retStatus, out.strip(), err)

    def submit(self, filepath, verify=False):
        submitCmd = "submit "
        Cmd = self.rhcertCmd + submitCmd + filepath
        (exitStatus, out, err) = utilities.runShellCommand(
                                    Cmd, rhcertCommand=True
                                )
        #TODO items
        return tests

    def save(self, testname, verify=False):
        retStatus = True
        saveCmd = "save "
        Cmd = self.rhcertCmd + saveCmd 
        (exitStatus, out, err) = utilities.runShellCommand(
                                    Cmd, rhcertCommand=False
                                )
        if not os.path.exists('/tmp/hwcert_report'):
            os.makedirs('/tmp/hwcert_report')
        # for python2-3 compatibility
        if isinstance(out, bytes):
            out = out.decode()
        if not 'no test results to save' in out:
            log_file_path = out.split(':')[1].split('\n')[1]
            log_file_name = os.path.basename(log_file_path)
            new_log_file_name = testname + '_' + log_file_name
            logs_dir = '/tmp/hwcert_report/' + new_log_file_name
            shutil.move(log_file_path, logs_dir)
            ret = self.check_exception(logs_dir)
        return retStatus

    def check_exception(self, filename):
        retStatus = True
        exception_out_file = '/tmp/hwcert_report/' + 'exceptions.out'
        exception_log_file = '/tmp/hwcert_report/' + 'exceptions.log'
        command = "zgrep -ni -E -A 20 'Exception raised|Traceback' " + filename
        found = utilities.runShellCommand(command, rhcertCommand=False)[1]
        if isinstance(found, bytes):
            found = found.decode()
        if len(found) > 0:
            retStatus = False
            print('Exception found in file {}'.format(filename))
            if not os.path.exists(exception_out_file):
                os.mknod(exception_out_file)
            with open(exception_out_file, 'a') as f:
                f.write(filename + "\n")
            if not os.path.exists(exception_log_file):
                os.mknod(exception_log_file)
            with open(exception_log_file, 'a') as f:
                f.write(filename + "\n")
                f.write("=============================================\n")
                f.write(found + "\n")
        return retStatus

    def clean(self, verify=True):
        retStatus = True
        cleanCmds = "clean"
        Cmd = self.rhcertCmd + cleanCmds
        (exitStatus, out, err) = utilities.runShellCommand(
                                    Cmd, rhcertCommand=True
                                )
        #TODO items
        return retStatus

    def cleanall(self, verify=True):
        cleanallCmd = "clean all"
        Cmd = self.rhcertCmd + cleanallCmd 
        (exitStatus, out, err) = utilities.runShellCommand(
                                    Cmd, rhcertCommand=True
                                )
        #TODO items
        return True

    def report(self, testname, verify=True):
        retStatus = True
        if self.runDetails:
            for _, v in self.runDetails.items():
                # CERTPX-5904, now tests showing as 'REVIEW'
                # will be marked as 'PASS' as well
                if v[0] == testname and v[-1] == 'FAIL':
                    retStatus = False
        else:
            retStatus = False
        return retStatus

    def register(self, lts_server, verify=True):
        retStatus = True
        registerCmd = "register --server "
        # register with given server
        Cmd = self.rhcertCmd + registerCmd + lts_server
        (exitStatus, out, err) = utilities.runShellCommand(
                                    Cmd, rhcertCommand=True
                                )
        #TODO items
        return retStatus
