from hwcert import HWCert
import pytest
import time
import os
import sys
from pprint import pprint

cert = HWCert()
ret = cert.cleanall()
# Verify is LTS Server is available else exit
lts_server = os.getenv('LTS_SERVER', 'dummy')
if lts_server != 'dummy':
    ret = cert.register(lts_server)
else:
    print('LTS server name not found...Exiting')
    sys.exit(1)

# Check planned tests
ret, plannedTests, plannedArgs = cert.plan()

def printPlanned(testname):
    print("\n{} is not planned on this setup\n".format(testname))
    print("\nPlanned tests include \n")
    pprint(plannedTests)
    print("\nPlanned tests raw output \n")
    pprint(cert.checkPlannedRaw)

@pytest.fixture(scope='session')
def hwdriver(request):
    driver = HWCert()
    driver.plan()
    ret, setup_info, err = driver.setup_info()
    if not ret:
        pytest.exit("\n\n{}\n\n".format(err))
    print('setup_info : {}'.format(setup_info))
    return driver

# 100MegEthernet test
testname_100MEthernet = '100MegEthernet'
@pytest.mark.hwcert_test_ethernet
def test_run_hwcert_100MEthernet(hwdriver):
    if not testname_100MEthernet in plannedTests:
        printPlanned(testname_100MEthernet)
        raise pytest.skip()
    ret, out, err = hwdriver.run(testname_100MEthernet, device=True, devicename=hwdriver.planArgs[testname_100MEthernet][0][0])
    assert ret == True

# 1GigEthernet test
testname_1GEthernet = '1GigEthernet'
@pytest.mark.hwcert_test_ethernet
def test_run_hwcert_1GEthernet(hwdriver):
    if not testname_1GEthernet in plannedTests:
        printPlanned(testname_1GEthernet)
        raise pytest.skip()
    ret, out, err = hwdriver.run(testname_1GEthernet, device=True, devicename=hwdriver.planArgs[testname_1GEthernet][0][0])
    assert ret == True

# 2.5GigEthernet test
testname_2_5GEthernet = '2.5GigEthernet'
@pytest.mark.hwcert_test_ethernet
def test_run_hwcert_2_5GEthernet(hwdriver):
    if not testname_2_5GEthernet in plannedTests:
        printPlanned(testname_2_5GEthernet)
        raise pytest.skip()
    ret, out, err = hwdriver.run(testname_2_5GEthernet, device=True, devicename=hwdriver.planArgs[testname_2_5GEthernet][0][0])
    assert ret == True

# 5GigEthernet test
testname_5GEthernet = '5GigEthernet'
@pytest.mark.hwcert_test_ethernet
def test_run_hwcert_5GEthernet(hwdriver):
    if not testname_5GEthernet in plannedTests:
        printPlanned(testname_5GEthernet)
        raise pytest.skip()
    ret, out, err = hwdriver.run(testname_5GEthernet, device=True, devicename=hwdriver.planArgs[testname_5GEthernet][0][0])
    assert ret == True

# 10GigEthernet test
testname_10GEthernet = '10GigEthernet'
@pytest.mark.hwcert_test_ethernet
def test_run_hwcert_10GEthernet(hwdriver):
    if not testname_10GEthernet in plannedTests:
        printPlanned(testname_10GEthernet)
        raise pytest.skip()
    ret, out, err = hwdriver.run(testname_10GEthernet, device=True, devicename=hwdriver.planArgs[testname_10GEthernet][0][0])
    assert ret == True

# 20GigEthernet test
testname_20GEthernet = '20GigEthernet'
@pytest.mark.hwcert_test_ethernet
def test_run_hwcert_20GEthernet(hwdriver):
    if not testname_20GEthernet in plannedTests:
        printPlanned(testname_20GEthernet)
        raise pytest.skip()
    ret, out, err = hwdriver.run(testname_20GEthernet, device=True, devicename=hwdriver.planArgs[testname_20GEthernet][0][0])
    assert ret == True

# 25GigEthernet test
testname_25GEthernet = '25GigEthernet'
@pytest.mark.hwcert_test_ethernet
def test_run_hwcert_25GEthernet(hwdriver):
    if not testname_25GEthernet in plannedTests:
        printPlanned(testname_25GEthernet)
        raise pytest.skip()
    ret, out, err = hwdriver.run(testname_25GEthernet, device=True, devicename=hwdriver.planArgs[testname_25GEthernet][0][0])
    assert ret == True

# 40GigEthernet test
testname_40GEthernet = '40GigEthernet'
@pytest.mark.hwcert_test_ethernet
def test_run_hwcert_40GEthernet(hwdriver):
    if not testname_40GEthernet in plannedTests:
        printPlanned(testname_40GEthernet)
        raise pytest.skip()
    ret, out, err = hwdriver.run(testname_40GEthernet, device=True, devicename=hwdriver.planArgs[testname_40GEthernet][0][0])
    assert ret == True

# 50GigEthernet test
testname_50GEthernet = '50GigEthernet'
@pytest.mark.hwcert_test_ethernet
def test_run_hwcert_50GEthernet(hwdriver):
    if not testname_50GEthernet in plannedTests:
        printPlanned(testname_50GEthernet)
        raise pytest.skip()
    ret, out, err = hwdriver.run(testname_50GEthernet, device=True, devicename=hwdriver.planArgs[testname_50GEthernet][0][0])
    assert ret == True

# 100GigEthernet test
testname_100GEthernet = '100GigEthernet'
@pytest.mark.hwcert_test_ethernet
def test_run_hwcert_100GEthernet(hwdriver):
    if not testname_100GEthernet in plannedTests:
        printPlanned(testname_100GEthernet)
        raise pytest.skip()
    ret, out, err = hwdriver.run(testname_100GEthernet, device=True, devicename=hwdriver.planArgs[testname_100GEthernet][0][0])
    assert ret == True

# add-on test
testname_exception = 'exception'
@pytest.mark.hwcert_test_non_int
def test_run_hwcert_exception(hwdriver):
    exception_out_file = '/tmp/hwcert_report/' + 'exceptions.out'
    exception_log_file = '/tmp/hwcert_report/' + 'exceptions.log'
    ret = True
    if os.path.exists(exception_out_file) or os.path.exists(exception_log_file):
        print("Python Exceptions found in log files, please check exceptions.out and exceptions.log")
        ret = False
    assert ret == True
