from hwcert import HWCert
import pytest
import time
import os
from pprint import pprint

cert = HWCert()
ret, plannedTests, plannedArgs = cert.plan()

def printPlanned(testname):
    print("\n{} is not planned on this setup\n".format(testname))
    print("\nPlanned tests include \n")
    pprint(plannedTests)
    print("\nPlanned tests raw output \n")
    pprint(cert.checkPlannedRaw)

@pytest.fixture(scope='session')
def hwdriver(request):
    driver = HWCert()
    driver.plan()
    ret, setup_info, err = driver.setup_info()
    if not ret:
        pytest.exit("\n\n{}\n\n".format(err))
    print('setup_info : {}'.format(setup_info))
    return driver

# default test
testname_info = 'info'
@pytest.mark.hwcert_test_non_int
def test_run_hwcert_info(hwdriver):
    if not testname_info in plannedTests:
        printPlanned(testname_info)
        raise pytest.skip()
    ret, out, err = hwdriver.run(testname_info)
    assert ret == True

# default test
testname_self_check = 'self_check'
@pytest.mark.hwcert_test_non_int
def test_run_hwcert_self_check(hwdriver):
    if not testname_self_check in plannedTests:
        printPlanned(testname_self_check)
        raise pytest.skip()
    ret, out, err = hwdriver.run(testname_self_check)
    assert ret == True

# default test
testname_sosreport = 'sosreport'
@pytest.mark.hwcert_test_non_int
def test_run_hwcert_sosreport(hwdriver):
    if not testname_sosreport in plannedTests:
        printPlanned(testname_sosreport)
        raise pytest.skip()
    ret, out, err = hwdriver.run(testname_sosreport)
    assert ret == True

# non-interactive test
testname_memory = 'memory'
@pytest.mark.hwcert_test_non_int
def test_run_hwcert_memory(hwdriver):
    if not testname_memory in plannedTests:
        printPlanned(testname_memory)
        raise pytest.skip()
    ret, out, err = hwdriver.run(testname_memory)
    assert ret == True

# non-interactive test
testname_core = 'core'
@pytest.mark.hwcert_test_non_int
def test_run_hwcert_core(hwdriver):
    if not testname_core in plannedTests:
        printPlanned(testname_core)
        raise pytest.skip()
    ret, out, err = hwdriver.run(testname_core)
    assert ret == True

# non-interactive test
testname_cpuscaling = 'cpuscaling'
@pytest.mark.hwcert_test_non_int
def test_run_hwcert_cpuscaling(hwdriver):
    if not testname_cpuscaling in plannedTests:
        printPlanned(testname_cpuscaling)
        raise pytest.skip()
    ret, out, err = hwdriver.run(testname_cpuscaling)
    assert ret == True

# non-interactive test
testname_fv_core = 'fv_core'
@pytest.mark.hwcert_test_non_int
def test_run_hwcert_fv_core(hwdriver):
    if not testname_fv_core in plannedTests:
        printPlanned(testname_fv_core)
        raise pytest.skip()
    ret, out, err = hwdriver.run(testname_fv_core)
    assert ret == True

# non-interactive test
testname_fv_memory = 'fv_memory'
@pytest.mark.hwcert_test_non_int
def test_run_hwcert_fv_memory(hwdriver):
    if not testname_fv_memory in plannedTests:
        printPlanned(testname_fv_memory)
        raise pytest.skip()
    ret, out, err = hwdriver.run(testname_fv_memory)
    assert ret == True

# non-interactive test
testname_fv_cpu_pinning = 'fv_cpu_pinning'
@pytest.mark.hwcert_test_non_int
def test_run_hwcert_fv_cpu_pinning(hwdriver):
    if not testname_fv_cpu_pinning in plannedTests:
        printPlanned(testname_fv_cpu_pinning)
        raise pytest.skip()
    ret, out, err = hwdriver.run(testname_fv_cpu_pinning)
    assert ret == True

# non-interactive test
testname_hw_profiler = 'hw_profiler'
@pytest.mark.hwcert_test_non_int
def test_run_hwcert_hw_profiler(hwdriver):
    if not testname_hw_profiler in plannedTests:
        printPlanned(testname_hw_profiler)
        raise pytest.skip()
    ret, out, err = hwdriver.run(testname_hw_profiler)
    assert ret == True

# non-interactive test
testname_sw_profiler = 'sw_profiler'
@pytest.mark.hwcert_test_non_int
def test_run_hwcert_sw_profiler(hwdriver):
    if not testname_sw_profiler in plannedTests:
        printPlanned(testname_sw_profiler)
        raise pytest.skip()
    ret, out, err = hwdriver.run(testname_sw_profiler)
    assert ret == True

# storage test, requires device
testname_storage = 'STORAGE'
@pytest.mark.hwcert_test_non_int
def test_run_hwcert_storage(hwdriver):
    if not testname_storage in plannedTests:
        printPlanned(testname_storage)
        raise pytest.skip()
    ret, out, err = hwdriver.run(testname_storage, device=True, devicename=hwdriver.planArgs[testname_storage][0][0])
    assert ret == True

# SATA test, requires device
testname_sata = 'SATA'
@pytest.mark.hwcert_test_non_int
def test_run_hwcert_sata(hwdriver):
    if not testname_sata in plannedTests:
        printPlanned(testname_sata)
        raise pytest.skip()
    ret, out, err = hwdriver.run(testname_sata, device=True, devicename=hwdriver.planArgs[testname_sata][0][0])
    assert ret == True

# SATA_SSD test, requires device
testname_sata_ssd = 'SATA_SSD'
@pytest.mark.hwcert_test_non_int
def test_run_hwcert_sata_ssd(hwdriver):
    if not testname_sata_ssd in plannedTests:
        printPlanned(testname_sata_ssd)
        raise pytest.skip()
    ret, out, err = hwdriver.run(testname_sata_ssd, device=True, devicename=hwdriver.planArgs[testname_sata_ssd][0][0])
    assert ret == True

# SAS test, requires device
testname_sas = 'SAS'
@pytest.mark.hwcert_test_non_int
def test_run_hwcert_sas(hwdriver):
    if not testname_sas in plannedTests:
        printPlanned(testname_sas)
        raise pytest.skip()
    ret, out, err = hwdriver.run(testname_sas, device=True, devicename=hwdriver.planArgs[testname_sas][0][0])
    assert ret == True

# add-on test
testname_exception = 'exception'
@pytest.mark.hwcert_test_non_int
def test_run_hwcert_exception(hwdriver):
    exception_out_file = '/tmp/hwcert_report/' + 'exceptions.out'
    exception_log_file = '/tmp/hwcert_report/' + 'exceptions.log'
    ret = True
    if os.path.exists(exception_out_file) or os.path.exists(exception_log_file):
        print("Python Exceptions found in log files, please check exceptions.out and exceptions.log")
        ret = False
    assert ret == True
