from hwcert import HWCert
import pytest
import time
import os
from pprint import pprint

cert = HWCert()
ret, plannedTests, plannedArgs = cert.plan()

def printPlanned(testname):
    print("\n{} is not planned on this setup\n".format(testname))
    print("\nPlanned tests include \n")
    pprint(plannedTests)
    print("\nPlanned tests raw output \n")
    pprint(cert.checkPlannedRaw)

@pytest.fixture(scope='session')
def hwdriver(request):
    driver = HWCert()
    driver.plan()
    ret, setup_info, err = driver.setup_info()
    if not ret:
        pytest.exit("\n\n{}\n\n".format(err))
    print('setup_info : {}'.format(setup_info))
    return driver

# realtime test
testname_realtime = 'realtime'
@pytest.mark.hwcert_test_non_int
def test_run_hwcert_realtime(hwdriver):
    if not testname_realtime in plannedTests:
        printPlanned(testname_realtime)
        raise pytest.skip()
    ret, out, err = hwdriver.run(testname_realtime)
    assert ret == True

## fv_realtime test, needs reboot, hence commented
#testname_fv_realtime = 'fv_realtime'
#@pytest.mark.hwcert_test_non_int
#def test_run_hwcert_fv_realtime(hwdriver):
#    if not testname_fv_realtime in plannedTests:
#        printPlanned(testname_fv_realtime)
#        raise pytest.skip()
#    ret, out, err = hwdriver.run(testname_fv_realtime)
#    assert ret == True

# add-on test
testname_exception = 'exception'
@pytest.mark.hwcert_test_non_int
def test_run_hwcert_exception(hwdriver):
    exception_out_file = '/tmp/hwcert_report/' + 'exceptions.out'
    exception_log_file = '/tmp/hwcert_report/' + 'exceptions.log'
    ret = True
    if os.path.exists(exception_out_file) or os.path.exists(exception_log_file):
        print("Python Exceptions found in log files, please check exceptions.out and exceptions.log")
        ret = False
    assert ret == True
